using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using DataDynamics.Analysis.Core.Schema;
using DataDynamics.Analysis.Core.DataProvider;
using DataDynamics.Analysis.Core.Data;
using DataDynamics.Analysis.Services.Undo;
using DataDynamics.Analysis.DataViewModel;

namespace SampleSchemaBuilder
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private static Schema PrepareSchema()
        {
            SchemaBuilder sb = new SchemaBuilder("TestingPubsTitles");
            sb.SetDataProviderName("UNBOUND").SetConnectionString("unbound").SetQueryString("unbound");

            /*add query fields to the schema; these are the fields used to query the database and referenced by key expressions
             * for attribute and measure fields
            */
            sb.AddQueryFields(
                "title_id", "title", "type", "pub_id", "price", "advance",
                "royalty", "ytd_sales", "notes", "pubdate_str", "au_id", "au_lname", "au_fname",
                //the following query fields setup fields to be used for getting parts of a date, these could also be considered "calculated fields"
                new SchemaBuilder.QueryField("PubDate")
                    .SetExpression<DateTime>("=CDate(Fields!pubdate_str.Value)"),
                new SchemaBuilder.QueryField("PubDate_Year")
                    .SetExpression<int>("=Year(Fields!PubDate.Value)"),
                new SchemaBuilder.QueryField("PubDate_Quarter")
                    .SetExpression<int>("=DatePart(DateInterval.Quarter, Fields!PubDate.Value)"),
                new SchemaBuilder.QueryField("PubDate_Month")
                    .SetExpression<int>("=Month(Fields!PubDate.Value)"),
                new SchemaBuilder.QueryField("PubDate_Day")
                    .SetExpression<int>("=Day(Fields!PubDate.Value)")
                );

            //setup some temporary variables to store fields and dimensions
            SchemaBuilder.MeasuresDimension tmpMeasureDimension;
            SchemaBuilder.AttributesDimension tmpAttributeDimension;
            SchemaBuilder.AttributeField tmpAttributeField;
            SchemaBuilder.MeasureField tmpMeasureField;
            SchemaBuilder.HierarchyField tmpHierarchy;
            SchemaBuilder.Level tmpLevel;

            /*setup the measures dimension
             * this is where all the measure fields should be placed
            */
            tmpMeasureDimension = new SchemaBuilder.MeasuresDimension("Measures");

            /* create a measure field
             * set the key expression specifying the type (<int>) and where it gets its data (values of the title_id query Field)
             * set the aggregate function to count, this typically defaults to sum.
             * set the format the data will be displayed in
             * add the field to the dimension
             */
            tmpMeasureField = new SchemaBuilder.MeasureField("Order count")
                .SetAggregateFunction(SchemaBuilder.AggregateFunction.Count)
                .SetKeyExpression<int>("=Fields!title_id.Value")
                .SetDefaultFormat("#0");
            tmpMeasureDimension.AddField(tmpMeasureField);

            /* create a measure field
             * set the key expression specifying the type (<float>) and where it gets its data (values of the Price query Field)
             * set the aggregate function to average, this typically defaults to sum.
             * add the field to the dimension
             */
            tmpMeasureField = new SchemaBuilder.MeasureField("Unit Price")
                .SetKeyExpression<float>("=Fields!price.Value")
                .SetAggregateFunction(SchemaBuilder.AggregateFunction.Avg);         
            tmpMeasureDimension.AddField(tmpMeasureField);

            /* create a measure field
             * set the key expression specifying the type (<float>) and where it gets its data (values of the Price query Field)
             * set the locale to ensure American formats are applied
             * set the default format to currency using the "c"
             * add the field to the dimension
             */
            tmpMeasureField = new SchemaBuilder.MeasureField("Total Cost")
                .SetKeyExpression<float>("=Fields!price.Value")
                .SetLocale("en-US")
                .SetDefaultFormat("c");
            tmpMeasureDimension.AddField(tmpMeasureField);

            tmpMeasureField = new SchemaBuilder.MeasureField("Advance")
                .SetKeyExpression<float>("=Fields!advance.Value");
            tmpMeasureDimension.AddField(tmpMeasureField);

            tmpMeasureField = new SchemaBuilder.MeasureField("Royalty")
                .SetKeyExpression<float>("=Fields!royalty.Value");
            tmpMeasureDimension.AddField(tmpMeasureField);

            //add the measure dimension to the schema builder
            sb.AddDimension(tmpMeasureDimension);

            /* create a new attributes dimension called "Other Fields"
             * This dimension will group 3 fields together
             */ 
            tmpAttributeDimension = new SchemaBuilder.AttributesDimension("Other Fields");
                    
            /* Create an AttributeField
             * set the field's caption
             * set the key expression specifying the type (<string>) and where it gets its data (values of the title query field)
             * set the field as the default for the dimension (required for all attributes dimensions
             * add the field to the dimension
             */
            tmpAttributeField = new SchemaBuilder.AttributeField("title")
                .SetCaption("Book Title")
                .SetKeyExpression<string>("=Fields!title.Value");
            tmpAttributeDimension.SetDefaultField(tmpAttributeField);
            tmpAttributeDimension.AddField(tmpAttributeField);

            tmpAttributeField = new SchemaBuilder.AttributeField("type")
                .SetCaption("Type")
                .SetKeyExpression<string>("=Fields!type.Value");
            tmpAttributeDimension.AddField(tmpAttributeField);

            /* Create an AttributeField
             * set the field's caption
             * set the key expression spedifying the type (<string>) and where it gets its data (values of the au_name query field)
             * set the caption expression to display the authors name as last name first first name last
             * add the field to the dimension
             */        
            tmpAttributeField = new SchemaBuilder.AttributeField("author")
                .SetCaption("Author name")
                .SetKeyExpression<string>("=Fields!au_id.Value")
                .SetCaptionExpression("=Fields!au_lname.Value & \" \" & Fields!au_fname.Value");
            tmpAttributeDimension.AddField(tmpAttributeField);
   
            //add the dimension to the schema builder 
            sb.AddDimension(tmpAttributeDimension);

            /*create another attributes dimension
             * this dimension will group Date fields together
             */ 
            tmpAttributeDimension = new SchemaBuilder.AttributesDimension("Date");
            
            //create a hierarchy field to use as a Date Hierarchy
            tmpHierarchy = new SchemaBuilder.HierarchyField("Pub Date");

            tmpLevel = new SchemaBuilder.Level("Year")
                .SetKeyExpression<DateTime>("=Fields!PubDate_Year.Value")
                .SetCaptionExpression("=Fields!PubDate_Year.Value");
            tmpHierarchy.AddLevel(tmpLevel);

            tmpLevel = new SchemaBuilder.Level("Quarter")
                .SetKeyExpression<int>("=Fields!PubDate_Quarter.Value")
                .SetCaptionExpression("=\"Q\" & Fields!PubDate_Quarter.Value & \" \" & Fields!PubDate_Year.Value");
            tmpHierarchy.AddLevel(tmpLevel);

            tmpLevel = new SchemaBuilder.Level("Month")
                .SetKeyExpression<int>("=Fields!PubDate_Month.Value")
                .SetCaptionExpression("=MonthName(Fields!PubDate_Month.Value) & \", \" & Fields!PubDate_Year.Value");
            tmpHierarchy.AddLevel(tmpLevel);

            tmpLevel = new SchemaBuilder.Level("Day")
                .SetKeyExpression<int>("=Fields!PubDate_Day.Value")
                .SetCaptionExpression("=MonthName(Fields!PubDate_Month.Value) & \" \" & Fields!PubDate_Day.Value & \", \" & Fields!PubDate_Year.Value");
            tmpHierarchy.AddLevel(tmpLevel);

            //add the hierarchy and create normal attribute fields that specifiy the same Key Expressions
            tmpAttributeDimension.AddFields(
                tmpHierarchy,
                new SchemaBuilder.AttributeField("Date").SetKeyExpression<DateTime>("=Fields!PubDate.Value"),
                new SchemaBuilder.AttributeField("Year").SetKeyExpression<int>("=Fields!PubDate_Year.Value"),
                new SchemaBuilder.AttributeField("Quarter").SetKeyExpression<int>("=Fields!PubDate_Quarter.Value"),
                new SchemaBuilder.AttributeField("Month").SetKeyExpression<int>("=Fields!PubDate_Month.Value"),
                new SchemaBuilder.AttributeField("Day").SetKeyExpression<int>("=Fields!PubDate_Day.Value")
            );
            //set the hierarchy as the default field
            tmpAttributeDimension.SetDefaultField(tmpHierarchy);

            //add the field to the schema builder
            sb.AddDimension(tmpAttributeDimension);

            //validate the schema
            sb.Validate();
            return sb.GetSchema();
        }

        private static DataSet PrepareDataSet()
        {
            //setup the same dataset used in the In-Memory dataset sample
            DataSet dataset = new DataSet("TestingPubsTitles");
            DataTable table = dataset.Tables.Add("TestingPubsTitles");
            table.Columns.Add("title_id", typeof(string));
            table.Columns.Add("title", typeof(string));
            table.Columns.Add("type", typeof(string));
            table.Columns.Add("pub_id", typeof(string));
            table.Columns.Add("price", typeof(decimal));
            table.Columns.Add("advance", typeof(decimal));
            table.Columns.Add("royalty", typeof(int));
            table.Columns.Add("ytd_sales", typeof(int));
            table.Columns.Add("notes", typeof(string));
            table.Columns.Add("pubdate_str", typeof(string));
            table.Columns.Add("au_id", typeof(string));
            table.Columns.Add("au_lname", typeof(string));
            table.Columns.Add("au_fname", typeof(string));

            table.Rows.Add("BU1032", "The Busy Executive's Database Guide", "business    ", "1389", 19.99, 5000.00, 10, 4095, "An overview of available database systems with emphasis on common business applications. Illustrated.", new DateTime(1991, 06, 12), "409-56-7008", "Green", "Marjorie");
            table.Rows.Add("BU1032", "The Busy Executive's Database Guide", "business    ", "1389", 19.99, 5000.00, 10, 4095, "An overview of available database systems with emphasis on common business applications. Illustrated.", new DateTime(1991, 06, 12), "409-56-7008", "Bennet", "Abraham");
            table.Rows.Add("BU1111", "Cooking with Computers: Surreptitious Balance Sheets", "business    ", "1389", 11.95, 5000.00, 10, 3876, "Helpful hints on how to use your electronic resources to the best advantage.", new DateTime(1991, 06, 09), "267-41-2394", "O'Leary", "Michael");
            table.Rows.Add("BU1111", "Cooking with Computers: Surreptitious Balance Sheets", "business    ", "1389", 11.95, 5000.00, 10, 3876, "Helpful hints on how to use your electronic resources to the best advantage.", new DateTime(1991, 06, 09), "724-80-9391", "MacFeather", "Stearns");
            table.Rows.Add("BU2075", "You Can Combat Computer Stress!", "business    ", "0736", 2.99, 10125.00, 24, 18722, "The latest medical and psychological techniques for living with the electronic office. Easy-to-understand explanations.", new DateTime(1991, 06, 30), "213-46-8915", "Green", "Marjorie");
            table.Rows.Add("BU7832", "Straight Talk About Computers", "business    ", "1389", 19.99, 5000.00, 10, 4095, "Annotated analysis of what computers can do for you: a no-hype guide for the critical user.", new DateTime(1991, 06, 22), "274-80-9391", "Straight", "Dean");
            table.Rows.Add("MC2222", "Silicon Valley Gastronomic Treats", "mod_cook    ", "0877", 19.99, 0.00, 12, 2032, "\"Favorite recipes for quick, easy and elegant meals.\"", new DateTime(1991, 06, 09), "712-45-1867", "del Castillo", "Innes");
            table.Rows.Add("MC3021", "The Gourmet Microwave", "mod_cook    ", "0877", 2.99, 15000.00, 24, 22246, "Traditional French gourmet recipes adapted for modern microwave cooking.", new DateTime(1991, 06, 18), "722-51-5454", "DeFrance", "Michel");
            table.Rows.Add("MC3021", "The Gourmet Microwave", "mod_cook    ", "0877", 2.99, 15000.00, 24, 22246, "Traditional French gourmet recipes adapted for modern microwave cooking.", new DateTime(1991, 06, 18), "899-46-2035", "Ringer", "Anne");
            table.Rows.Add("PC1035", "But Is It User Friendly?", "popular_comp", "1389", 22.95, 7000.00, 16, 8780, "\"A survey of software for the naive user focusing on the 'friendliness' of each.\"", new DateTime(1991, 06, 30), "238-95-7766", "Carson", "Cheryl");
            table.Rows.Add("PC8888", "Secrets of Silicon Valley", "popular_comp", "1389", 20.00, 8000.00, 10, 4095, "Muckraking reporting on the world's largest computer hardware and software manufacturers.", new DateTime(1994, 06, 12), "427-17-2319", "Dull", "Ann");
            table.Rows.Add("PC8888", "Secrets of Silicon Valley", "popular_comp", "1389", 20.00, 8000.00, 10, 4095, "Muckraking reporting on the world's largest computer hardware and software manufacturers.", new DateTime(1994, 06, 12), "846-92-7186", "Hunter", "Sheryl");
            table.Rows.Add("PC9999", "Net Etiquette", "popular_comp", "1389", null, null, null, null, "A must-read for computer conferencing.", new DateTime(2000, 08, 06), "486-29-1786", "Locksley", "Charlene");
            table.Rows.Add("PS1372", "Computer Phobic AND Non-Phobic Individuals: Behavior Variations", "psychology  ", "0877", 21.59, 7000.00, 10, 375, "\"A must for the specialist, this book examines the difference between those who hate and fear computers and those who don't.", new DateTime(1991, 10, 21), "724-80-9391", "MacFeather", "Stearns");
            table.Rows.Add("PS1372", "Computer Phobic AND Non-Phobic Individuals: Behavior Variations", "psychology  ", "0877", 21.59, 7000.00, 10, 375, "\"A must for the specialist, this book examines the difference between those who hate and fear computers and those who don't.", new DateTime(1991, 10, 21), "756-30-7391", "Karsen", "Livia");
            table.Rows.Add("PS2091", "Is Anger the Enemy?", "psychology  ", "0736", 10.95, 2275.00, 12, 2045, "Carefully researched study of the effects of strong emotions on the body. Metabolic charts included.", new DateTime(1991, 06, 15), "899-46-2035", "Ringer", "Anne");
            table.Rows.Add("PS2091", "Is Anger the Enemy?", "psychology  ", "0736", 10.95, 2275.00, 12, 2045, "Carefully researched study of the effects of strong emotions on the body. Metabolic charts included.", new DateTime(1991, 06, 15), "998-72-3567", "Ringer", "Albert");
            table.Rows.Add("PS2106", "Life Without Fear", "psychology  ", "0736", 7.00, 6000.00, 10, 111, "\"New exercise, meditation, and nutritional techniques that can reduce the shock of daily interactions. Popular audience. Sample menus included exercise video available separately.\"", new DateTime(1991, 10, 05), "998-72-3567", "Ringer", "Albert");
            table.Rows.Add("PS3333", "Prolonged Data Deprivation: Four Case Studies", "psychology  ", "0736", 19.99, 2000.00, 10, 4072, "What happens when the data runs dry?  Searching evaluations of information-shortage effects.", new DateTime(1991, 06, 12), "172-32-1176", "White", "Johnson");
            table.Rows.Add("PS7777", "Emotional Security: A New Algorithm", "psychology  ", "0736", 7.99, 4000.00, 10, 3336, "Protecting yourself and your loved ones from undue emotional stress in the modern world. Use of computer and nutritional aids emphasized.", new DateTime(1991, 06, 12), "486-29-1786", "Locksley", "Charlene");
            table.Rows.Add("TC3218", "\"Onions, Leeks, and Garlic: Cooking Secrets of the Mediterranean\"", "trad_cook   ", "0877", 20.95, 7000.00, 10, 375, "\"Profusely illustrated in color, this makes a wonderful gift book for a cuisine-oriented friend.", new DateTime(1991, 10, 21), "807-91-6654", "Panteley,Sylvia");
            table.Rows.Add("TC4203", "Fifty Years in Buckingham Palace Kitchens", "trad_cook   ", "0877", 11.95, 4000.00, 14, 15096, "\"More anecdotes from the Queen's favorite cook describing life among English royalty. Recipes techniques, tender vignettes.\"", new DateTime(1991, 06, 12), "648-92-1872", "Blotchet-Halls", "Reginald");
            table.Rows.Add("TC7777", "\"Sushi, Anyone?\"", "trad_cook   ", "0877", 14.99, 8000.00, 10, 4095, "Detailed instructions on how to make authentic Japanese sushi in your spare time.", new DateTime(1991, 06, 12), "267-41-2394", "O'Leary", "Michael");
            table.Rows.Add("TC7777", "\"Sushi, Anyone?\"", "trad_cook   ", "0877", 14.99, 8000.00, 10, 4095, "Detailed instructions on how to make authentic Japanese sushi in your spare time.", new DateTime(1991, 06, 12), "472-27-2349", "Gringlesby", "Burt");
            table.Rows.Add("TC7777", "\"Sushi, Anyone?\"", "trad_cook   ", "0877", 14.99, 8000.00, 10, 4095, "Detailed instructions on how to make authentic Japanese sushi in your spare time.", new DateTime(1991, 06, 12), "672-71-3249", "Yokomoto", "Akiko");

            return dataset;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.Text = pivotView1.ProductName + " " + pivotView1.ProductVersion + " In-Memory Dataset Sample";
            PivotDataProvider dataProvider = pivotView1.DataProvider;
            DataSet dataset = PrepareDataSet();
            Schema schema = PrepareSchema();

            //connect the pivotview's dataprovider to the schema and the dataset
            dataProvider.Connect<DataSet>(schema, dataset);
            dataProvider.CubeName = "TestingPubsTitles";

            //get a refernce to the pivotView's intenral view definition
            ViewDefinition model = pivotView1.InternalModel;
            //use the transaction to configure the view
            using (ITransaction transaction = model.CreateTransaction("Initial layout inject"))
            {
                model.MarkingType = MarkingType.Circle; //set the marking to the circle type
                //add the Pub Date Hierarchy and Total Cost Measure tot he column shelf
                model.ColumnShelf.Add(model.CreateField(FieldTypes.Attribute, "[Date].[Pub Date]"));
                model.ColumnShelf.Add(model.CreateField(FieldTypes.Measure, "[Measures].[Total Cost]"));
                //add the Author attribute field to the row shelf
                model.RowShelf.Add(model.CreateField(FieldTypes.Attribute, "[Other Fields].[author]"));
                //add the order count measure field to the color shelf, validate the model, and commit the transaction to the view
                model.ColorShelf.Field = model.CreateField(FieldTypes.Measure, "[Measures].[Order count]");
                model.Validate();
                transaction.Commit();
            }

        }

    }
}